@@ -22,7 +22,7 @@
 #	times=NULL, dose=NULL, ccov=NULL, tvcov=NULL, nest=NULL, torder=0,
 #	interaction=NULL, transform="identity", link="identity",
 #	autocorr="exponential", pell=NULL, preg=NULL, pvar=var(y), varfn=NULL,
-#	par=NULL, pre=NULL, delta=NULL, shfn=FALSE, common=FALSE,
+#	covfn=NULL, par=NULL, pre=NULL, delta=NULL, shfn=FALSE, common=FALSE,
 #	envir=parent.frame(), print.level=0, ndigit=10,
 #	gradtol=0.00001, steptol=0.00001, iterlim=100, fscale=1,
 #	stepmax=10*sqrt(theta%*%theta), typsiz=abs(c(theta)))
@@ -37,7 +37,7 @@ elliptic <- function(response=NULL, model="linear", distribution="normal",
 	times=NULL, dose=NULL, ccov=NULL, tvcov=NULL, nest=NULL,
 	torder=0, interaction=NULL, transform="identity",
 	link="identity", autocorr="exponential", pell=NULL, preg=NULL,
-	pvar=var(y), varfn=NULL, par=NULL, pre=NULL, delta=NULL,
+	covfn=NULL, pvar=var(y), varfn=NULL, par=NULL, pre=NULL, delta=NULL,
 	shfn=FALSE, common=FALSE, envir=parent.frame(),
 	print.level=0, ndigit=10, gradtol=0.00001, steptol=0.00001,
 	iterlim=100, fscale=1, stepmax=10*sqrt(theta%*%theta),
@@ -48,6 +48,7 @@ elliptic <- function(response=NULL, model="linear", distribution="normal",
 plra <- function(theta){
 	if(mdl==2)mu <- mu1(theta)
 	if(cvar==1)varn <- sh1(theta)
+	if(!is.null(covfn))covn <- cv1(theta)
 	z <- .Fortran("plra",
 		theta=as.double(theta),
 		like=double(1),
@@ -73,6 +74,7 @@ plra <- function(theta){
 		nccov=as.integer(nccov),
 		npvar=as.integer(npvar),
 		cvar=as.integer(cvar),
+		ccvar=as.integer(!is.null(covfn)),
 		npre=as.integer(npre),
 		npar=as.integer(npar),
 		link=as.integer(lnk),
@@ -89,6 +91,7 @@ plra <- function(theta){
 		tb=double(npvl),
 		as.double(mu),
 		as.double(varn),
+		as.double(covn),
 		DUP=FALSE,
 		PACKAGE="growth")
 	list(like=z$like,res=z$dev,beta=z$beta,betacov=z$betacov)}
@@ -98,6 +101,7 @@ plra <- function(theta){
 plral <- function(theta){
 	if(mdl==2)mu <- mu1(theta)
 	if(cvar==1)varn <- sh1(theta)
+	if(!is.null(covfn))covn <- cv1(theta)
 	z <- .Fortran("plra",
 		theta=as.double(theta),
 		like=double(1),
@@ -123,6 +127,7 @@ plral <- function(theta){
 		nccov=as.integer(nccov),
 		npvar=as.integer(npvar),
 		cvar=as.integer(cvar),
+		ccvar=as.integer(!is.null(covfn)),
 		npre=as.integer(npre),
 		npar=as.integer(npar),
 		link=as.integer(lnk),
@@ -139,6 +144,7 @@ plral <- function(theta){
 		tb=double(npvl),
 		as.double(mu),
 		as.double(varn),
+		as.double(covn),
 		DUP=FALSE,
 		PACKAGE="growth")
 	z$like}
@@ -294,12 +300,16 @@ times <- resp$response$times
 nobs <- nobs(resp)
 nind <- length(nobs)
 nld <- max(nobs)
-mu <- varn <- NULL
+mu <- varn <- covn <- NULL
 npre <- length(pre)
+if(!is.null(covfn)){
+	if(max(nobs)!=2)
+		stop("covfn can only be used with two observations per individual")
+	if(npre==0)stop("initial estimates mus tbe provide for covfn")}
 #
 # if a data object was supplied, modify formulae or functions to read from it
 #
-mu3 <- sh3 <- NULL
+mu3 <- sh3 <- cv3<- NULL
 if(respenv||inherits(envir,"repeated")||inherits(envir,"tccov")||inherits(envir,"tvcov")){
 	if(inherits(model,"formula")){
 		mu3 <- if(respenv)finterp(model,.envir=response,.name=envname)
@@ -322,7 +332,18 @@ if(respenv||inherits(envir,"repeated")||inherits(envir,"tccov")||inherits(envir,
 		        	else fnenvir(varfn,.envir=envir,.name=envname)
 		        sh3 <- varfn
 		        attr(sh3,"model") <- tmp}
-		else sh3 <- varfn}}
+		else sh3 <- varfn}
+	if(inherits(covfn,"formula")){
+		cv3 <- if(respenv)finterp(covfn,.envir=response,.name=envname)
+			else finterp(covfn,.envir=envir,.name=envname)}
+	else if(is.function(covfn)){
+		if(is.null(attr(covfn,"model"))){
+		        tmp <- parse(text=deparse(covfn)[-1])
+		        covfn <- if(respenv)fnenvir(covfn,.envir=response,.name=envname)
+		        	else fnenvir(covfn,.envir=envir,.name=envname)
+		        cv3 <- covfn
+		        attr(cv3,"model") <- tmp}
+		else cv3 <- covfn}}
 npr <- length(preg)
 #
 # transform model formula to function and check number of parameters
@@ -393,6 +414,8 @@ npvar <- length(pvar)
 n1 <- if(common){if(inherits(varfn,"formula"))
 	length(attr(mu1,"parameters"))+1 else 1} else length(preg)+1
 n2 <- length(c(preg,pvar))
+n3 <- n2+1
+n4 <- length(c(preg,pvar,pre))
 #
 # transform variance formula to function and check number of parameters
 #
@@ -459,6 +482,65 @@ if(common){
 	nlp <- length(unique(c(attr(mu1,"parameters"),attr(sh1,"parameters"))))
 	if(nlp!=npr)stop(paste("with a common parameter model, preg should contain",nlp,"estimates"))}
 #
+# transform covariance formula to function and check number of parameters
+#
+if(inherits(covfn,"formula")){
+	cv3 <- if(respenv)finterp(covfn,.envir=response,.start=n3,.name=envname)
+		else finterp(covfn,.envir=envir,.start=n3,.name=envname)
+	tmp <- attributes(cv3)
+	cv2 <- cv3
+	attributes(cv2) <- tmp
+	npt3 <- length(attr(cv2,"parameters"))
+	if(is.character(attr(cv2,"model"))){
+	# W&R formula
+		if(length(attr(cv2,"model"))==1){
+			cv1 <- function(p) p[n3]*rep(1,n)
+			attributes(cv1) <- attributes(cv2)
+			cv2 <- NULL}}
+	else {
+	# formula with unknowns
+		if(npre!=npt3){
+			cat("\nParameters are ")
+			cat(attr(cv2,"parameters"),"\n")
+			stop(paste("pre should have",npt3,"estimates"))}
+		if(is.list(pre)){
+			if(!is.null(names(pre))){
+				o <- match(attr(cv2,"parameters"),names(pre))
+				pre <- unlist(pre)[o]
+				if(sum(!is.na(o))!=length(pre))stop("invalid estimates for covfn - probably wrong names")}
+			else pre <- unlist(pre)}}
+	if(!is.null(cv2)){
+		if(inherits(envir,"tccov")){
+			cv <- covind(response)
+			cv1 <- function(p) cv2(p)[cv]
+			attributes(cv1) <- attributes(cv2)}
+		else {
+			cv1 <- cv2
+			rm(cv2)}}}
+else if(is.function(covfn))cv1 <- function(p) covfn(p[n3:n4])
+else cv1 <- NULL
+#
+# give appropriate attributes to cv1 for printing
+#
+if(!is.null(cv1)&&is.null(attr(cv1,"parameters")))
+	attributes(cv1) <- if(is.function(covfn)){
+		if(!inherits(covfn,"formulafn")){
+			if(respenv)attributes(fnenvir(covfn,.envir=response))
+			else attributes(fnenvir(covfn,.envir=envir))}
+		else attributes(covfn)}
+		else {
+			if(respenv)attributes(fnenvir(cv1,.envir=response))
+			else attributes(fnenvir(cv1,.envir=envir))}
+nlp3 <- if(is.function(covfn))length(attr(cv1,"parameters"))
+	else if(is.null(covfn)||is.character(covfn))NULL
+	else if(!inherits(covfn,"formula"))stop("covfn must be a function or formula")
+	else npt3
+#
+# if possible, check that correct number of estimates was supplied
+#
+if(!is.null(nlp3)&&nlp3!=npre)
+	stop(paste("pre should have",nlp3,"initial estimates"))
+#
 # check interactions of covariates with times
 #
 if(mdl==1&&!is.null(interaction)){
@@ -500,7 +582,7 @@ if(full){
 		for(j in 1:dim(resp$ccov$ccov)[2])
 			tmp <- tmp+resp$ccov$ccov[i,j]*2^(j-1)
 		rxl <- c(rxl,tmp)}
-	nxrl <- max(rxl)
+	nxrl <- length(unique(rxl)) #max(rxl)
 	p <- preg}
 else {
 	nxrl <- 1
@@ -573,11 +655,19 @@ if(!(mdl==4&&npvar==4)&&cvar==0){
 	pvar <- log(pvar)}
 theta <- c(preg,pvar)
 #
+# set up covariance function if there is one and check initial estimates
+#
+if(is.function(cv1)){
+	if(length(cv1(pre))!=sum(nobs(resp)))
+		stop("The covariance function or formula must provide an estimate for each observation")}
+#
 # check AR and random effect
 #
 if(npre>0){
-	if(any(pre<=0))stop("All variance components must be positive")
-	theta <- c(theta,log(pre))}
+	if(is.null(covfn)){
+		if(any(pre<=0))stop("All variance components must be positive")
+		theta <- c(theta,log(pre))}
+	else theta <- c(theta,pre)}
 if(npar>0){
 	if(par<=0||(par>=1&&ar!=5))
 		stop("Estimate of autocorrelation must lie between 0 and 1")
@@ -601,7 +691,7 @@ pred <- y-z$res
 #
 sigsq <- p[(npv+1):(npv+npvar)]
 if(!(mdl==4&&npvar==4)&&!common)sigsq <- exp(sigsq)
-if(npre>0)tausq <- exp(p[(npv+npvar+1):(npv+npvar+npre)])
+if(npre>0&&is.null(covfn))tausq <- exp(p[(npv+npvar+1):(npv+npvar+npre)])
 else tausq <- 0
 if(npar>0){
 	rho <- exp(p[npv+npvar+npre+1])
@@ -681,14 +771,17 @@ if(dst==1&&!common&&length(sigsq)==1&&((npar>0&&
 else rpred <- sdr <- rres <- NULL
 if(!is.null(mu3))mu1 <- mu3
 if(!is.null(sh3))sh1 <- sh3
+if(!is.null(cv3))cv1 <- cv3
 z <- list(
 	call=call,
 	model=model,
 	distribution=distribution,
 	mu1=mu1,
 	varfn=varfn,
+	covfn=covfn,
 	common=common,
 	sh1=sh1,
+	cv1=cv1,
 	shfn=shfn,
 	autocorr=autocorr,
 	response=resp$response,
@@ -900,13 +993,32 @@ else {
 	dimnames(coef.table) <- list(vname,c("estimate","se","sigsq"))
 	print.default(coef.table,digits=digits,print.gap=2)}
 if(z$npre>0){
-	cat("\nVariance components\n")
-	coef.table <- cbind(z$coef[(z$npv+z$npvar+1):(z$npv+z$npvar+z$npre)],
-		z$nlse[(z$npv+z$npvar+1):(z$npv+z$npvar+z$npre)],z$tausq)
-	if(z$npre==1)cname <- "tausq"
-	else cname <- c("Level 1","Level 2")
-	dimnames(coef.table) <- list(cname,c("estimate","se",""))
-	print.default(coef.table,digits=digits,print.gap=2)}
+	if(is.null(z$covfn)){
+		cat("\nVariance components\n")
+		coef.table <- cbind(z$coef[(z$npv+z$npvar+1):(z$npv+z$npvar+z$npre)],
+			z$nlse[(z$npv+z$npvar+1):(z$npv+z$npvar+z$npre)],z$tausq)
+		if(z$npre==1)cname <- "tausq"
+		else cname <- c("Level 1","Level 2")
+		dimnames(coef.table) <- list(cname,c("estimate","se",""))
+		print.default(coef.table,digits=digits,print.gap=2)}
+	else if(inherits(z$cv1,"formulafn")){
+		cat("\nCovariance function parameters\n")
+		cname <- NULL
+		if(!is.null(attr(z$cv1,"formula")))
+			cat(deparse(attr(z$cv1,"formula")),sep="\n")
+		else if(!is.null(attr(z$cv1,"model"))){
+			t <- deparse(attr(z$cv1,"model"))
+			t[1] <- sub("expression\\(","",t[1])
+			t[length(t)] <- sub("\\)$","",t[length(t)])
+			cat(t,sep="\n")}
+		cname <- c(cname,if(is.character(attr(z$cv1,"model")))
+				attr(z$cv1,"model")
+			else if(is.expression(attr(z$cv1,"model")))
+				attr(z$cv1,"parameters")
+			else attr(z$cv1,"parameters")[1:z$npre])
+		coef.table <- cbind(z$coef[(z$npv+z$npvar+1):z$np], z$nlse[(z$npv+z$npvar+1):z$np])
+		dimnames(coef.table) <- list(unique(cname),c("estimate","se"))
+		print.default(coef.table,digits=digits,print.gap=2)}}
 if(z$rho!=0){
 	cat("\n",z$autocorr," autocorrelation\n",sep="")
 	coef.table <- cbind(z$coef[z$npv+z$npvar+z$npre+1],
