@@ -58,7 +58,7 @@ fcn <- fcn[!(fcn%in%mem)]
 if(length(mem)>0){
 	tmp <- vector(length=length(mem))
 	for(i in 1:length(mem))
-		tmp[i] <- exists(mem[i],envir=envir)&&is.function(eval(parse(text=mem[i]),envir=envir))&&(length(grep(paste(mem[i],","),ch))>0||length(grep(paste(",",mem[i]),ch))>0||length(grep(paste("=",mem[i]),ch))>0||length(grep(paste("\\(",mem[i],"\\)"),ch))>0)&&mem[i]!="times"&&mem[i]!="response"
+		tmp[i] <- exists(mem[i],envir=envir)&&is.function(eval(parse(text=mem[i]),envir=envir))&&(length(grep(paste(mem[i],","),ch))>0||length(grep(paste(",",mem[i]),ch))>0||(length(grep(paste("=",mem[i]),ch))>0&&length(grep(paste(">=",mem[i]),ch))==0&&length(grep(paste("<=",mem[i]),ch))==0)||length(grep(paste("\\(",mem[i],"\\)"),ch))>0)&&mem[i]!="times"&&mem[i]!="response"
 	fcn <- unique(c(fcn,mem[tmp]))
 	mem <- mem[!tmp]}
 #
@@ -156,7 +156,7 @@ if(length(.mem)==0||all(.ex|.fac|.local)){
 			.dm <- matrix(1)
 			colnames(.dm) <- "(Intercept)"}
 		else {
-			.dm <- model.matrix(.mt,model.frame(.mt,.envir))
+			.dm <- model.matrix(.mt,model.frame(.mt,.envir,na.action=NULL))
 			if(!.intercept).dm <- .dm[,-1,drop=FALSE]}
 		.fna <- function(.p) as.vector(.dm%*%
 			.p[attr(.fna,"range")[1]:attr(.fna,"range")[2]])
@@ -193,6 +193,7 @@ if(.vector){
 	# check if some parameters are common to another formula
 	if(!is.null(.old)){
 		.j <- match(.pars,.un)
+		.j <- .j[!is.na(.j)]
 		.un <- .un[-.j]
 		.pars <- .pars[!is.na(.j)]
 		.range <- .range[!is.na(.j)]
@@ -355,7 +356,7 @@ if((is.null(.ex1)&&is.null(.ex2)&&is.null(.ex3)&&is.null(.ex4)&&is.null(.ex5)&&i
 			colnames(.dm) <- "(Intercept)"
 			.fna <- function(.p) rep(.p[attr(.fna,"range")[1]],.n)}
 		else {
-			.dm <- model.matrix(.mt,model.frame(.mt))
+			.dm <- model.matrix(.mt,model.frame(.mt,na.action=NULL))
 			if(!.intercept).dm <- .dm[,-1,drop=FALSE]
 			if(length(.ex1a)>0)for(.j in 1:length(.ex1a))
 				colnames(.dm) <- gsub(paste(.ndata,"\\$ccov\\$ccov\\[, ",.ex1a[.j],"\\]",sep=""),paste(colnames(.envir$ccov$ccov)[.ex1a[.j]],sep=""),colnames(.dm))
@@ -406,6 +407,7 @@ if(!is.null(.args)){
 #
 if(!is.null(.old)){
 	.j <- match(.pars,.un)
+	.j <- .j[!is.na(.j)]
 	.un <- .un[-.j]
 	.pars <- .pars[!is.na(.j)]
 	.range <- .range[!is.na(.j)]
@@ -508,7 +510,7 @@ if(is.null(.ex1)||all(!is.na(.ex1))){
 			colnames(.dm) <- "(Intercept)"
 			.fna <- function(.p) rep(.p[attr(.fna,"range")[1]],.n)}
 		else {
-			.dm <- model.matrix(.mt,model.frame(.mt))
+			.dm <- model.matrix(.mt,model.frame(.mt,na.action=NULL))
 			if(!.intercept).dm <- .dm[,-1,drop=FALSE]
 			if(length(.ex1a)>0)for(.j in 1:length(.ex1a))
 				colnames(.dm) <- gsub(paste(.ndata,.cc2,"\\$ccov\\[, ",.ex1a[.j],"\\]",sep=""),paste(.cn[.ex1a[.j]],sep=""),colnames(.dm))
@@ -553,6 +555,7 @@ if(!is.null(.args)){
 #
 if(!is.null(.old)){
 	.j <- match(.pars,.un)
+	.j <- .j[!is.na(.j)]
 	.un <- .un[-.j]
 	.pars <- .pars[!is.na(.j)]
 	.range <- .range[!is.na(.j)]
@@ -654,7 +657,7 @@ if(is.null(.ex1)||all(!is.na(.ex1))){
 			colnames(.dm) <- "(Intercept)"
 			.fna <- function(.p) rep(.p[attr(.fna,"range")[1]],.n)}
 		else {
-			.dm <- model.matrix(.mt,model.frame(.mt))
+			.dm <- model.matrix(.mt,model.frame(.mt,na.action=NULL))
 			if(!.intercept).dm <- .dm[,-1,drop=FALSE]
 			if(length(.ex1a)>0)for(.j in 1:length(.ex1a))
 				colnames(.dm) <- gsub(paste(.ndata,.cc2,"\\$ccov\\[, ",.ex1a[.j],"\\]",sep=""),paste(.cn[.ex1a[.j]],sep=""),colnames(.dm))
@@ -699,6 +702,7 @@ if(!is.null(.args)){
 #
 if(!is.null(.old)){
 	.j <- match(.pars,.un)
+	.j <- .j[!is.na(.j)]
 	.un <- .un[-.j]
 	.pars <- .pars[!is.na(.j)]
 	.range <- .range[!is.na(.j)]
@@ -791,7 +795,7 @@ if(is.null(.ex1)||all(!is.na(.ex1))){
 			colnames(.dm) <- "(Intercept)"
 			.fna <- function(.p) rep(.p[attr(.fna,"range")[1]],.n)}
 		else {
-			.dm <- model.matrix(.mt,model.frame(.mt,data=.envir))
+			.dm <- model.matrix(.mt,model.frame(.mt,data=.envir,na.action=NULL))
 			if(!.intercept).dm <- .dm[,-1,drop=FALSE]
 			.fna <- function(.p) as.vector(.dm%*%
 				.p[attr(.fna,"range")[1]:attr(.fna,"range")[2]])}
@@ -832,6 +836,7 @@ if(!is.null(.args)){
 #
 if(!is.null(.old)){
 	.j <- match(.pars,.un)
+	.j <- .j[!is.na(.j)]
 	.un <- .un[-.j]
 	.pars <- .pars[!is.na(.j)]
 	.range <- .range[!is.na(.j)]
