@@ -118,6 +118,26 @@ if(is.list(linear)){
 	lin1 <- linear[[1]]
 	lin2 <- linear[[2]]}
 else lin1 <- linear
+if(inherits(lin1,"formula")&&is.null(mu)){
+	mu <- lin1
+	lin1 <- NULL}
+if(inherits(lin2,"formula")&&is.null(shape)){
+	shape <- lin2
+	lin2 <- NULL}
+if(inherits(lin1,"formula")){
+	lin1model <- if(respenv){
+		if(!is.null(attr(finterp(lin1,.envir=y,.name=envname),"parameters")))
+			attr(finterp(lin1,.envir=y,.name=envname),"model")}
+	else {if(!is.null(attr(finterp(lin1,.envir=envir,.name=envname),"parameters")))
+			attr(finterp(lin1,.envir=envir,.name=envname),"model")}}
+else lin1model <- NULL
+if(inherits(lin2,"formula")){
+	lin2model <- if(respenv){
+		if(!is.null(attr(finterp(lin2,.envir=y,.name=envname),"parameters")))
+			attr(finterp(lin2,.envir=y,.name=envname),"model")}
+	else {if(!is.null(attr(finterp(lin2,.envir=envir,.name=envname),"parameters")))
+			attr(finterp(lin2,.envir=envir,.name=envname),"model")}}
+else lin2model <- NULL
 #
 # if a data object was supplied, modify formulae or functions to read from it
 #
@@ -641,6 +661,7 @@ z1 <- list(
 	mix=NULL,
 	censor=censor,
 	linear=list(lin1,lin2),
+	linmodel=list(lin1model,lin2model),
 	common=common,
 	maxlike=z0$minimum,
 	fitted.values=fitted.values,
