@@ -29,6 +29,103 @@ df.residual.gnlm <- function(z) z$df
 
 deviance.gnlm <- function(z) 2*z$maxlike
 
+coef.gnlm <- function(z){
+gnlmm <- !is.null(z$points)
+mix <- z$npm>0||!is.null(z$mix)
+npl <- z$npl-gnlmm
+if(npl>0){
+	if(z$distribution=="own"){
+		cname <- if(is.character(attr(z$likefn,"model")))
+			attr(z$likefn,"model")
+		else if(length(grep("linear",attr(z$likefn,"parameters")))>0)
+		attr(z$likefn,"parameters")[grep("\\[",attr(z$likefn,"parameters"))]
+		else attr(z$likefn,"parameters")}
+	else {
+		cname <- if(is.character(attr(z$mu,"model")))attr(z$mu,"model")
+		else if(length(grep("linear",attr(z$mu,"parameters")))>0)
+		attr(z$mu,"parameters")[grep("\\[",attr(z$mu,"parameters"))]
+		else attr(z$mu,"parameters")}
+	if(!is.null(z$linmodel[[1]]))cname <-
+	c(cname,z$linmodel[[1]])}
+if(gnlmm)cname <- c(cname," ")
+if(z$npm>0&&!is.null(z$mix)){
+	cname <- c(cname,if(is.character(attr(z$mix,"model")))
+		attr(z$mix,"model")
+		else if(length(grep("linear",attr(z$mix,"parameters")))>0)
+		attr(z$mix,"parameters")[grep("\\[",attr(z$mix,"parameters"))]
+		else attr(z$mix,"parameters"))
+	if(!is.null(z$linmodel[[2]]))cname <- c(cname,z$linmodel[[2]])}
+if(z$common||z$nps>0){
+	if(!is.null(z$shape))cname <- c(cname,if(mix&&!gnlmix)" "
+		else if(is.character(attr(z$shape,"model")))
+			attr(z$shape,"model")
+		else if(length(grep("linear",attr(z$shape,"parameters")))>0||
+		length(grep("mu",attr(z$shape,"parameters")))>0)
+		attr(z$shape,"parameters")[grep("\\[",attr(z$shape,"parameters"))]
+		else attr(z$shape,"parameters"))
+	if(!is.null(z$linmodel[[2]]))cname <- c(cname,z$linmodel[[2]])}
+if(z$npf>0||!is.null(z$family)){
+	cname <- c(cname,if(is.character(attr(z$family,"model")))
+		attr(z$family,"model")
+		else if(length(grep("linear",attr(z$family,"parameters")))>0)
+		attr(z$family,"parameters")[grep("\\[",attr(z$family,"parameters"))]
+		else attr(z$family,"parameters"))
+	if(!is.null(z$linmodel[[3]]))cname <- c(cname,z$linmodel[[3]])}
+if(z$common)cname <- unique(cname)
+if(mix)cname <- c(cname," ")
+coef <- z$coef
+names(coef) <- cname
+coef}
+
+vcov.gnlm <- function(z){
+gnlmm <- !is.null(z$points)
+mix <- z$npm>0||!is.null(z$mix)
+npl <- z$npl-gnlmm
+if(npl>0){
+	if(z$distribution=="own"){
+		cname <- if(is.character(attr(z$likefn,"model")))
+			attr(z$likefn,"model")
+		else if(length(grep("linear",attr(z$likefn,"parameters")))>0)
+		attr(z$likefn,"parameters")[grep("\\[",attr(z$likefn,"parameters"))]
+		else attr(z$likefn,"parameters")}
+	else {
+		cname <- if(is.character(attr(z$mu,"model")))attr(z$mu,"model")
+		else if(length(grep("linear",attr(z$mu,"parameters")))>0)
+		attr(z$mu,"parameters")[grep("\\[",attr(z$mu,"parameters"))]
+		else attr(z$mu,"parameters")}
+	if(!is.null(z$linmodel[[1]]))cname <-
+	c(cname,z$linmodel[[1]])}
+if(gnlmm)cname <- c(cname," ")
+if(z$npm>0&&!is.null(z$mix)){
+	cname <- c(cname,if(is.character(attr(z$mix,"model")))
+		attr(z$mix,"model")
+		else if(length(grep("linear",attr(z$mix,"parameters")))>0)
+		attr(z$mix,"parameters")[grep("\\[",attr(z$mix,"parameters"))]
+		else attr(z$mix,"parameters"))
+	if(!is.null(z$linmodel[[2]]))cname <- c(cname,z$linmodel[[2]])}
+if(z$common||z$nps>0){
+	if(!is.null(z$shape))cname <- c(cname,if(mix&&!gnlmix)" "
+		else if(is.character(attr(z$shape,"model")))
+			attr(z$shape,"model")
+		else if(length(grep("linear",attr(z$shape,"parameters")))>0||
+		length(grep("mu",attr(z$shape,"parameters")))>0)
+		attr(z$shape,"parameters")[grep("\\[",attr(z$shape,"parameters"))]
+		else attr(z$shape,"parameters"))
+	if(!is.null(z$linmodel[[2]]))cname <- c(cname,z$linmodel[[2]])}
+if(z$npf>0||!is.null(z$family)){
+	cname <- c(cname,if(is.character(attr(z$family,"model")))
+		attr(z$family,"model")
+		else if(length(grep("linear",attr(z$family,"parameters")))>0)
+		attr(z$family,"parameters")[grep("\\[",attr(z$family,"parameters"))]
+		else attr(z$family,"parameters"))
+	if(!is.null(z$linmodel[[3]]))cname <- c(cname,z$linmodel[[3]])}
+if(z$common)cname <- unique(cname)
+if(mix)cname <- c(cname," ")
+vcov <- z$cov
+rownames(vcov) <- cname
+colnames(vcov) <- cname
+vcov}
+
 ### print function for bnlr, gnlr, gnlr3, gnlmm, and fmr
 ###
 print.gnlm <- function(z,digits=max(4,.Options$digits-3),correlation=TRUE) {
