@@ -168,19 +168,37 @@ qstable <- function(q, loc=0, disp=1/sqrt(2), skew=0, tail=2, eps=1.0e-6){
 		if(tail[i]==1&&skew[i]==0)tmp[i] <- qcauchy(q[i],loc[i],disp[i])
 		else if(tail[i]==2)tmp[i] <- qnorm(q[i],loc[i],disp[i]*sqrt(2))
 		else {
-			interval <- if(tail[i]<1&&abs(skew[i])==1)
-					sign(skew[i])*c(0.001,10)
-				else loc[i]+disp[i]*c(-5,5)
-			while(h(interval[1])*h(interval[2])>0)
-				interval <- 2*interval
-			tmp[i] <- uniroot(h,interval)$root}}
+			if(tail[i]<1&&abs(skew[i])==1){
+				interval <- sign(skew[i])*c(0.001,10)
+				while(TRUE){
+					tmp1 <- h(interval[1])
+					tmp2 <- h(interval[2])
+					if(is.na(tmp1*tmp2)||tmp1*tmp2<0)break
+					interval[1] <- 0.5*interval[1]
+					interval[2] <- 2*interval[2]}}
+			else {
+				interval <- loc[i]+disp[i]*c(-5,5)
+				while(TRUE){
+					tmp1 <- h(interval[1])
+					tmp2 <- h(interval[2])
+					if(is.na(tmp1*tmp2)||tmp1*tmp2<0)break
+					interval <- 2*interval}}
+			tmp[i] <- if(is.na(tmp1*tmp2))NA
+				  else uniroot(h,interval)$root}}
 	tmp}
 
 ###########################################################################
 # Generation of stable random deviates
 #
-rstable <- function(n=1,loc=0,disp=1/sqrt(2),skew=0,tail=2,eps=1.0e-6)
-	qstable(runif(n),loc=loc,disp=disp,skew=skew,tail=tail,eps=eps)
+rstable <- function(n=1,loc=0,disp=1/sqrt(2),skew=0,tail=2,eps=1.0e-6){
+	tmp1 <- qstable(runif(n),loc=loc,disp=disp,skew=skew,tail=tail,eps=eps)
+	tmp2 <- tmp1[!is.na(tmp1)]
+	n1 <- n
+	while(any(is.na(tmp1))){
+		n1 <- n1-sum(!is.na(tmp1))
+		tmp1 <- qstable(runif(n1),loc=loc,disp=disp,skew=skew,tail=tail,eps=eps)
+		tmp2 <- c(tmp2,tmp1[!is.na(tmp1)])}
+	tmp2}
 
 ###########################################################################
 # Stable hazard
