@@ -20,8 +20,8 @@ c  SYNOPSIS
 c
 c     subroutine plra(theta,like,iist,rxl,x,y,tvcov,ccov,dose,nobs,
 c    +     nbs,nest,lnest,dev,nind,nld,nxrl,np,npell,npv,npvl,
-c    +     nccov,npvar,cvar,npre,npar,link,torder,inter,model,ar,
-c    +     tvc,beta,betacov,v,sigsq,ey,tb,mu,var)
+c    +     nccov,npvar,cvar,ccvar,npre,npar,link,torder,inter,model,
+c    +     ar,tvc,beta,betacov,v,sigsq,ey,tb,mu,var,covar)
 c
 c  DESCRIPTION
 c
@@ -32,20 +32,22 @@ c
 c
       subroutine plra(theta,like,dist,rxl,x,y,tvcov,ccov,dose,nobs,
      +     nbs,nest,lnest,dev,nind,nld,nxrl,np,npell,npv,npvl,
-     +     nccov,npvar,cvar,npre,npar,link,torder,inter,model,ar,
-     +     tvc,beta,betacov,v,sigsq,ey,tb,mu,var)
+     +     nccov,npvar,cvar,ccvar,npre,npar,link,torder,inter,model,
+     +     ar,tvc,beta,betacov,v,sigsq,ey,tb,mu,var,covar)
 c
 c routine for computing -log(l) under the various models,
 c including both autoregression and (nested) random effects
 c
       implicit none
       integer np,nind,nxrl,link,i,j,k,nccov,tvc,ar,nbs,npell,dist,
-     +     lnest,nm,torder,model,npv,npvar,npre,npar,nld,npvl,cvar
+     +     lnest,nm,torder,model,npv,npvar,npre,npar,nld,npvl,cvar,
+     +     ccvar
       integer nobs(nind),inter(nccov),nest(1)
       double precision x(1),y(1),beta(npvl),betacov(npvl,npvl),
      +     tvcov(1),rxl(nind),dev(1),ccov(nind,nccov),ey(nld),
      +     like,theta(np),tb(npvl),v(nld,nld),tausq(2),sigsq(nld),
-     +     rho,ldet,qf,qf2,qf3,dose(nind),mu(1),var(1),pow,tmp1,tmp2
+     +     rho,ldet,qf,qf2,qf3,dose(nind),mu(1),var(1),pow,tmp1,tmp2,
+     +     covar(1)
 c
 c     compute parameters of the covariance matrix
 c
@@ -56,7 +58,7 @@ c
             sigsq(i)=tmp1
  1       continue
       endif
-      if(npre.gt.0)then
+      if(npre.gt.0.and.ccvar.eq.0)then
          do 3 i=1,npre
             if(theta(npv+npvar+i).gt.24)theta(npv+npvar+i)=24.
             tausq(i)=dexp(theta(npv+npvar+i))
@@ -83,7 +85,7 @@ c
 c estimate parameters in the linear model
 c
       if(model.eq.1)then
-         if(cvar.eq.0.and.link.eq.1.and.dist.ne.4)then
+         if(cvar.eq.0.and.ccvar.eq.0.and.link.eq.1.and.dist.ne.4)then
             call plml(x,y,beta,betacov,tb,ccov,tvcov,np,npv,npvl,
      +           nld,npvar,nind,nobs,nccov,torder,inter,tvc,v,
      +           sigsq,tausq,rho,nest,lnest,npar,npre,ar,nbs)
@@ -109,10 +111,14 @@ c
      +        nld,beta,npvl,inter,sigsq,cvar,var,tb,nbs,mu)
          endif
 c
+c     if necessary, compute covvariance function
+c
+         if(ccvar.eq.1)tausq(1)=covar(nm+1)
+c
 c     compute the inverse of variance matrix and its log determinant
 c
-         call cmpvar(v,ldet,sigsq,tausq,rho,nind,i,nm,x,nobs(i),nest,
-     +        lnest,nld,npre,npar,ar)
+         call cmpvar(v,ldet,sigsq,tausq,rho,nind,i,nm,x,nobs(i),
+     +        nest,lnest,nld,npre,npar,ar)
 c
 c     compute the mean function
 c
@@ -526,8 +532,8 @@ c
       end
 c
 c
-      subroutine cmpvar(v,ldet,sigsq,tausq,rho,nind,i,nm,x,nobs,nest,
-     +     lnest,nld,npre,npar,ar)
+      subroutine cmpvar(v,ldet,sigsq,tausq,rho,nind,i,nm,x,nobs,
+     +     nest,lnest,nld,npre,npar,ar)
 c
 c compute the variance matrix for the unit
 c
